<?php
/*
 * Plugin: Checkpoint_tracker_live 
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  »This is plugin for uaseco maniaplanet dedicated server controler. It is
 *  tracking and displaying detailed information about players cp positions and times
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
	
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
*/

	// Start the plugin
	$_PLUGIN = new Plugin_CheckPointTrackerlive();


/*
#///////////////////////////////////////////////////////////////////////#
#					#Plugin_CheckPointTrackerlive class
#///////////////////////////////////////////////////////////////////////#
*/

class Plugin_CheckPointTrackerlive extends Plugin {

	public $Config;
	public $Players = array();
	public $Displayed_players_num;
	public $Cp_color;
	public $Times_color;
	public $ML_pos_x;
	public $ML_pos_y;
	public $Map_cps;
	public $CtrlBgOpacity;
	public $TeamGameMode = false;
	public $Team1Color;
	public $Team2Color;


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#constructor function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function __construct () {

		$this->setAuthor('maxi031');
		$this->setVersion('1.1.3');
		$this->setBuild('2017-05-28');
		$this->setCopyright('2014 - 2017 by Mlade Gabic aka maxi031');
		$this->setDescription('Tracks and displays detailed information about players cp positions and times.');

		// Register functions for events
		$this->registerEvent('onSync', 'onSync');
		$this->registerEvent('onPlayerConnect', 'onPlayerConnect');
		$this->registerEvent('onPlayerDisconnect', 'onPlayerDisconnect');
		$this->registerEvent('onPlayerCheckpoint', 'onPlayerCheckpoint');
		$this->registerEvent('onPlayerInfoChanged', 'onPlayerInfoChanged');
		$this->registerEvent('onBeginMap', 'onBeginMap');
		$this->registerEvent('onPlayerFinish', 'onPlayerFinish');
		$this->registerEvent('onEndRound', 'onEndRound');
		$this->registerEvent('onLoadingMap', 'onLoadingMap');
		$this->registerEvent('onEndPlaying', 'onEndPlaying');
		$this->registerEvent('onBeginPodium', 'onBeginPodium');
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onSync function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onSync ($aseco) {

		// Read Configuration from xml file
		if (!$this->Config = $aseco->parser->xmlToArray('config/checkpoint_tracker_live.xml', true, true)) {
			trigger_error('[checkpoint_tracker_live] Could not read/parse config file "config/checkpoint_tracker_live.xml"!', E_USER_ERROR);
		}

		// Load settings from xml file into php variables
		$this->Config = $this->Config['CONFIG'];
		unset($this->Config['CONFIG']);
		$this->Displayed_players_num = $this->Config['DISPLAYED_PLAYERS_NUM'][0];
		$this->Cp_color = $this->Config['CHECKPOINTS_COLOR'][0];
		$this->Times_color = $this->Config['TIMES_COLOR'][0];
		$this->ML_pos_x = $this->Config['MANIALINK_POSITION_X'][0];
		$this->ML_pos_y = $this->Config['MANIALINK_POSITION_Y'][0];
		$this->CtrlBgOpacity = $this->Config['CTRL_BG_OPACITY'][0];
		$this->Team1Color = $this->Config['TEAM1_COLOR'][0];
		$this->Team2Color = $this->Config['TEAM2_COLOR'][0];
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onBeginPodium function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onBeginPodium ($aseco, $playerinfo) {
		// Create and send empty manialink
		$ml_cptracker = '<manialink version="3" id="CheckpointTrackerLive" name="CheckpointTrackerLive"></manialink>';
		$aseco->sendManialink($ml_cptracker, false, 0);
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onEndRound function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onEndRound ($aseco, $playerinfo) {

		// Clear cp positions of the players
		foreach ($this->Players as $pl) {
			if(isset($pl['playerLogin'])){
				$this->Players[$pl['playerLogin']]['currentCP'] = -1;
			}
		}

		// Create and send empty manialink
		$ml_cptracker = '<manialink version="3" id="CheckpointTrackerLive" name="CheckpointTrackerLive"></manialink>';
		$aseco->sendManialink($ml_cptracker, false, 0);
	}

	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onPlayerInfoChanged function
	#///////////////////////////////////////////////////////////////////////#
	*/


	public function onPlayerInfoChanged ($aseco, $login) {

		$gamestate = $aseco->server->gamestate;
		if($gamestate == Server::SCORE)//Bail out if there are scores dispalyed
			return;
		$player = $aseco->server->players->getPlayerByLogin((string)$login);
		if ($player->is_spectator){
			$this->Players[(string)$login]['currentCP'] = -1;
		}
		$this->Players[(string)$login]['teamID'] = $player->teamid;

		// Create and send manialink
		$ml_cptracker = $this->createManiaLink();
		$aseco->sendManialink($ml_cptracker, false, 0);
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onPlayerFinish function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onPlayerFinish ($aseco, $playerinfo) {

		$login = (string)$playerinfo->player->login;// make sure login is string

		// Continue only if player retires during rounds
		if ($playerinfo->score==0){
			if(($aseco->server->gameinfo->mode == Gameinfo::ROUNDS ||
			$aseco->server->gameinfo->mode == Gameinfo::TEAM))
			{
				$this->Players[$login]['currentCP'] += 1;	
			}
			else{
				$this->Players[$login]['currentCP'] += 1;
			}
			$ml_cptracker = $this->createManiaLink();
			$aseco->sendManialink($ml_cptracker, false, 0);
		}
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onLoadingMap function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onLoadingMap ($aseco, $map_item) {

		// Get number of cps from map item
		$this->Map_cps = ($map_item->nb_checkpoints);
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onBeginMap function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onBeginMap ($aseco, $data) {

		//Get current game mode
		$modescript['info'] = $aseco->client->query('GetModeScriptInfo');
		$scriptName = $modescript['info']['Name'];

		// Set correct teamgamemode
		if($scriptName == "Team.Script.txt" || $scriptName == "Chase.Script.txt"){
			$this->TeamGameMode = true;
		}
		else{
			$this->TeamGameMode = false;
		}

		// Create and send empty manialink widget
		$ml_cptracker = $this->createManiaLink(true);
		$aseco->sendManialink($ml_cptracker, false, 0);
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onEndPlaying function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onEndPlaying ($aseco, $data) {
		// Clear cp positions of the players
		foreach ($this->Players as $pl) {
			if(isset($pl['playerLogin'])){
				$this->Players[$pl['playerLogin']]['currentCP'] = -1;
			}
		}
		// Create and send invisible manialink
		$ml_cptracker = '<manialink version="3" id="CheckpointTrackerLive" name="CheckpointTrackerLive"></manialink>';
		$aseco->sendManialink($ml_cptracker, false, 0);
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onPlayerConnect function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onPlayerConnect ($aseco, $player) {

		$login = (string)$player->login;// make sure login is string
		$nick = $this->removeLinkFromString($player->nickname . "");
		$this->Players[$login]['playerLogin'] = $login;
		$this->Players[$login]['nickname'] = $aseco->handleSpecialChars($nick);
		$this->Players[$login]['isspectator'] = $player->is_spectator;
		$this->Players[$login]['currentCP'] = -1;
		$this->Players[$login]['arangeScore'] = 0;
		$this->Players[$login]['teamID'] = -1;

		// Create and send manialink to new player
		$ml_cptracker = $this->createManiaLink();
		$aseco->sendManialink($ml_cptracker, $login, 0);
	}

	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onPlayerCheckpoint function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onPlayerCheckpoint ($aseco, $params) {

		$login = (string)$params['login'];// make sure login is string

		// Check for multilap Maps and unsupported Gamemodes
		if ($aseco->server->gameinfo->mode == Gameinfo::TIME_ATTACK && $aseco->server->maps->current->multi_lap === true) {
			// Get current cp and time of the player
			$this->Players[$login]['currentCP'] = (int)$params['checkpoint_in_lap'];
			$this->Players[$login]['cpTime'] = $aseco->formatTime((int)$params['lap_time']);
			$this->Players[$login]['arangeScore'] = (int)$params['checkpoint_in_lap'] + (1 / (int)$params['lap_time']);
		}
		else
		{
			$this->Players[$login]['currentCP'] = (int)$params['checkpoint_in_race'];
			$this->Players[$login]['cpTime'] = $aseco->formatTime((int)$params['race_time']);
			$this->Players[$login]['arangeScore'] = (int)$params['checkpoint_in_race'] + (1 / (int)$params['race_time']);
		}

		// Sort array, create and send manialink
		$this->array_sort_by_column($this->Players, 'arangeScore');
		$ml_cptracker = $this->createManiaLink();
		$aseco->sendManialink($ml_cptracker, false, 0);
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#onPlayerDisconnect function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function onPlayerDisconnect ($aseco, $player) {

		// Ignore fluke disconnects with empty logins
		if ((string)$player->login == '') {
			return;
		}

		// Clear array and unset it
		$this->Players[(string)$player->login] = array();
		unset($this->Players[(string)$player->login]);

		// Create manialink
		$ml_cptracker = $this->createManiaLink();
		$aseco->sendManialink($ml_cptracker, false, 0);
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#array_sort_by_column function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function array_sort_by_column(&$arr, $col, $dir = SORT_DESC) {
		$sort_col = array();
	    foreach ($arr as $key=> $row) {
	    	if(isset($key) && isset($row[$col])){
	        	$sort_col[$key] = $row[$col];
	    	}
	    }
	    array_multisort($sort_col, $dir, $arr);
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#removeLinkFromString function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function removeLinkFromString($str)
	{
		$str = preg_replace('/\\$l\[(.*?)\]/is', ' ', $str);
		$str = preg_replace('/\\$l(.*?)\\$l/is', ' ', $str);
		return preg_replace('/\\$l/is', ' ', $str);
	}


	/*
	#///////////////////////////////////////////////////////////////////////#
	#					#createManialink function
	#///////////////////////////////////////////////////////////////////////#
	*/

	public function createManialink($empty=false){

		$ml_cptracker = '<manialink version="3" id="CheckpointTrackerLive" name="CheckpointTrackerLive">';
		$ml = '';
		$NumofCps = $this->Map_cps -1;
		$playersNumber = $this->Displayed_players_num;
		$ml_pos_x = $this->ML_pos_x;
		$ml_pos_y = $this->ML_pos_y;
		$Cp_color = $this->Cp_color;
		$Times_color = $this->Times_color;
		$Plugin_version = $this->getVersion();
		$CtrlBgOpacity = $this->CtrlBgOpacity;
		$T1Color = "";
		$T2Color = "";
		$i = 0;
		if($this->TeamGameMode){
			$T1Color = $this->Team1Color;
			$T2Color = $this->Team2Color;
		}
		// Put players info in manialink content
		if(!$empty)
		{
			// Get player cp and nickname for manialink
			foreach ($this->Players as $pl) {
				if(isset($pl['playerLogin'])){
					if($pl['currentCP'] != -1 ){
						$i++;
						$y_pos = 2+ (($i+1)*-2.8);
						$tempPlayerName =  $pl['nickname'];
						$tempcp = $this->Map_cps ==$pl['currentCP'] ? "FIN" : ($pl['currentCP']);
						$tempCpTime = $pl['cpTime'];
						$plogin= $pl['playerLogin'] ;
						$teamColor = $pl['teamID'] == 0 ? $T1Color : $T2Color;
						$ml .=	<<<EOD
							<label id="cpNum_$plogin" pos="0.5 $y_pos" z-index="3" size="7 2.5" textprefix="$Cp_color$teamColor" style="TextValueBig" halign="center" valign="top" autonewline="0" textsize="1" text="$tempcp"/>
							<label pos="4 $y_pos" z-index="3" textprefix="$Times_color$teamColor" style="TextValueBig" halign="left" valign="top" autonewline="0" textsize="1" text="$tempCpTime"/>
							<label id="$plogin" pos="19 $y_pos" z-index="2" size="26 2.5" scriptevents="1" style="TextValueBig" halign="left" valign="top" autonewline="0" textsize="1" text="$tempPlayerName"/>
EOD;
						// Exit the loop if we reached displayed players number limit
						if($i >= $playersNumber)
							break;
				}
			}
			}
		}

		// Create manialink
		$ml_cptracker .= 	<<<EOD

		<frame id="pctl_window" scale="1" pos="$ml_pos_x $ml_pos_y" z-index="1" hidden="0">
			<quad id="pctl_maximize" pos="-1.5 5" z-index="2" size="4 4" scriptevents="1" halign="left" valign="top" hidden="1" style="BgRaceScore2" substyle="SendScore"/>
			<frame id="pctl_hiddingPart" hidden="0">
				<quad id="pctl_icon_move" scriptevents="1" pos="24 5" z-index="2" size="4 4" hidden="1" halign="center" valign="top" style="UIConstructionBullet_Buttons" substyle="NewBullet"/>
				<quad pos="-1.5 5" z-index="2" size="4 4"  halign="left" valign="top" style="BgRaceScore2" substyle="ScoreLink"/>
				<quad id="pctl_cog" scriptevents="1" pos="41.5 5" z-index="2" size="4 4"  halign="left" valign="top" style="Icons128x32_1" substyle="Settings"/>
				<quad pos="-2 5" z-index="1" size="48 5" halign="left" valign="top" opacity="$CtrlBgOpacity" style="BgsPlayerCard" substyle="ProgressBar"/>
				<quad id="pctl_MarkLblBgd" pos="-1.8 25" z-index="1" size="48 3.5" halign="left" valign="top" hidden="1" style="BgsPlayerCard" substyle="ProgressBar"/>
				<label id="pctl_Title" pos="22.5 4" z-index="2" size="85 5" halign="center" valign="top" autonewline="0" maxline="1" scale="1" textsize="1" text="\$s\$fffLive cp tracker     \$ccaCPs: $NumofCps"/>
				<label pos="1 0" z-index="3" textprefix="\$s\$FD3" halign="center" valign="top" autonewline="0" textsize="1" text="Cp"/>
				<label pos="5 0" z-index="3" textprefix="\$s\$FD3" halign="left" valign="top" autonewline="0" textsize="1" text="Time"/>
				<label pos="19 0" z-index="2" textprefix="\$s\$FD3" halign="left" valign="top" autonewline="0" textsize="1" text="Player"/>
				$ml
			</frame>
		</frame>
		<frame id="pctl_cfg_menu" posn="0 0" z-index="7" scale= "1" hidden="1">
			<quad pos="0 0" z-index="6" opacity="$CtrlBgOpacity" size="20 5" halign="left" valign="top" style="BgsPlayerCard" substyle="ProgressBar" />
			<quad pos="0 -5" z-index="6" opacity="$CtrlBgOpacity" size="20 5" halign="left" valign="top" style="BgsPlayerCard" substyle="ProgressBar" />
			<label id="pctl_cfg_hide" pos="0 -1" z-index="7" size="33 2.5" scriptevents="1" style="TextValueBig" halign="left" valign="top" autonewline="0"  textsize="1" text="  Hide(Mimize)   "/>
			<label id="pctl_cfg_move" pos="0 -6" z-index="7" size="33 2.5" scriptevents="1" style="TextValueBig" halign="left" valign="top" autonewline="0"  textsize="1" text="  Move window   "/>
		</frame>
		<script><!--
		#Include "TextLib" as TextLib
		#Include "MathLib" as MathLib

		main()
		{
			declare Integer displayed_players_num = $playersNumber;
			declare CMlFrame pctl_window=(Page.GetFirstChild("pctl_window") as CMlFrame);
			declare CMlFrame pctl_cfg_menu=(Page.GetFirstChild("pctl_cfg_menu") as CMlFrame);
			declare CMlQuad pctl_maximize=(Page.GetFirstChild("pctl_maximize") as CMlQuad);
			declare CMlQuad pctl_icon_move=(Page.GetFirstChild("pctl_icon_move") as CMlQuad);
			declare CMlFrame pctl_hiddingPart=(Page.GetFirstChild("pctl_hiddingPart") as CMlFrame);
			declare persistent Boolean windowIsMoving for LocalUser;
			declare persistent Real windowPos_X for LocalUser; //-> this is by defalt 0
			declare persistent Real windowPos_Y for LocalUser;
			declare persistent Real cfgPos_X for LocalUser;
			declare persistent Real cfgPos_Y for LocalUser;
			declare Text currentScriptVersion = "$Plugin_version";
			declare persistent Text clienScriptVersion for LocalUser;
			declare persistent Boolean windowIsHidden for LocalUser;
			declare persistent Boolean menuIsOpen for LocalUser;

			// If script is firt time run set window position to default value
			if(currentScriptVersion != clienScriptVersion){
				clienScriptVersion = currentScriptVersion;
				windowPos_X = $ml_pos_x + 0.0;
				windowPos_Y = $ml_pos_y + 0.0;
				menuIsOpen = False;
			}

			// If player exist in tracker highlight him
			declare CMlLabel pctl_PlayerLabel = (Page.GetFirstChild("cpNum_" ^ LocalUser.Login) as CMlLabel);
			if(pctl_PlayerLabel != Null){
				declare CMlQuad pctl_MarkLblBgd = (Page.GetFirstChild("pctl_MarkLblBgd") as CMlQuad);
				pctl_MarkLblBgd.RelativePosition_V3.Y = pctl_PlayerLabel.RelativePosition_V3.Y + 0.5;
				pctl_MarkLblBgd.Show();
			}		

			pctl_window.RelativePosition_V3.X = windowPos_X;
			pctl_window.RelativePosition_V3.Y = windowPos_Y;

			// If menu was open open it again and tweak its position
			pctl_cfg_menu.Visible = menuIsOpen;
			pctl_cfg_menu.RelativePosition_V3.X = cfgPos_X;
			pctl_cfg_menu.RelativePosition_V3.Y = cfgPos_Y;

			//If window was hidden hide it again
			if(windowIsHidden){
				if(pctl_window.RelativePosition_V3.X > 0.0)
					pctl_maximize.RelativePosition_V3.X = 41.5;
				else
					pctl_maximize.RelativePosition_V3.X = -1.5;
				pctl_maximize.Visible  = True;
				pctl_hiddingPart.Visible  = False;
			}	

			if(windowIsMoving)
				pctl_icon_move.Visible = True;

			while(True){
				yield;
				if(windowIsMoving){
					pctl_window.RelativePosition_V3.X =MouseX - 23.5;
					pctl_window.RelativePosition_V3.Y =MouseY - 2.5;
				}
				foreach (Event in PendingEvents){
					if (Event.Type == CMlEvent::Type::MouseClick){
						if(Event.ControlId=="pctl_cfg_move"){
							//Move window here
							pctl_cfg_menu.Visible  = False;
							pctl_icon_move.Visible = True;
							windowIsMoving = True;
							menuIsOpen = False;	
						}
						else if(Event.ControlId=="pctl_cog"){
							if(pctl_window.RelativePosition_V3.X > 0.0){
								pctl_cfg_menu.RelativePosition_V3.X = MouseX - 16.0;
							}
							else{
								pctl_cfg_menu.RelativePosition_V3.X = MouseX;
							}
							pctl_cfg_menu.RelativePosition_V3.Y = MouseY - 2.0;
							pctl_cfg_menu.Visible  = !pctl_cfg_menu.Visible;
							cfgPos_X = pctl_cfg_menu.RelativePosition_V3.X;
							cfgPos_Y = pctl_cfg_menu.RelativePosition_V3.Y;
							menuIsOpen = pctl_cfg_menu.Visible;
						}
						else if(Event.ControlId=="pctl_cfg_hide"){
							if(pctl_window.RelativePosition_V3.X > 0.0)
								pctl_maximize.RelativePosition_V3.X = 41.5;
							else
								pctl_maximize.RelativePosition_V3.X = -1.5;
							pctl_hiddingPart.Visible  = False;
							pctl_maximize.Visible  = True;
							pctl_cfg_menu.Visible  = False;
							menuIsOpen = False;
							windowIsHidden = True;
						}
						else if(Event.ControlId=="pctl_maximize"){
							pctl_maximize.Visible  = False;
							pctl_hiddingPart.Visible  = True;
							windowIsHidden = False;
						}
						else if(Event.ControlId=="pctl_icon_move"){
							pctl_icon_move.Visible = False;
							windowIsMoving = False;

							// Tweak X position
							if(pctl_window.RelativePosition_V3.X <= -160.0)
								pctl_window.RelativePosition_V3.X  = -158.0;
							else if(pctl_window.RelativePosition_V3.X + 48.0 >= 160.0)
								pctl_window.RelativePosition_V3.X  = 112.0;

							// Tweak Y position
							if(pctl_window.RelativePosition_V3.Y >= 80.0)
								pctl_window.RelativePosition_V3.Y  = 78.0;
							else if(pctl_window.RelativePosition_V3.Y - 45.0 <= -90.0)
								pctl_window.RelativePosition_V3.Y  = -40.0;

							windowPos_X = pctl_window.RelativePosition_V3.X;
							windowPos_Y = pctl_window.RelativePosition_V3.Y;
						}
						else{
							pctl_cfg_menu.Visible = False;
							menuIsOpen = False;
							SetSpectateTarget(Event.ControlId);
						}
					}
				}
			}
		}
		--></script>

EOD;
		return $ml_cptracker . "</manialink>";
	}
}
?>